import { Component, OnInit, ViewChild } from '@angular/core';
import { Geolocation } from '@awesome-cordova-plugins/geolocation/ngx';
import { Platform } from '@ionic/angular';
import { Subscriber } from 'rxjs';
import { CategoriaPoi } from 'src/app/models';
import { GlobalService, PoiService } from 'src/app/services';
import { UtilityService } from 'src/app/services/utility/utility.service';
import { CATPOI, DELAYMAP, DURATIONTOAST, IMAGE, MESSAGES, SEARCH } from 'src/app/utility/global-constant';

declare function initMap(geolocation, showErrorToastFunc);
declare function geocodeRestSearch(searchStr, multi_markers_amenity);
declare function clearLayers();
declare function getWatchPositionRef(): Subscriber<any>;
declare function getLocationIntervalRef();

@Component({
	selector: 'app-poi',
	templateUrl: './poi.page.html',
	styleUrls: ['./poi.page.scss'],
})
export class PoiPage implements OnInit {
	@ViewChild('map') map: any;
	titleHeader: string = 'Mappa';
	logoHeader: string = IMAGE.logoHeader;
	widhtLogo: number = IMAGE.widthLogo;
	heightLogo: number = IMAGE.heightLogo;
	listaCategoriePoi: Array<CategoriaPoi>;
	listaCategoriePoiSave: Array<CategoriaPoi>;
	listaCategoriePoiFR: Array<CategoriaPoi>[];
	listaCategoriePoiFRR: Array<CategoriaPoi>[][];
	debounceSearch: number = SEARCH.debounce;
	categoriaSelected: CategoriaPoi;
	ios: boolean;
	android: boolean;
	tempSearch: string = 'xb';
	showMap: boolean = false;
	isLocalizationEnabled: boolean = false;
	constructor(
		private platform: Platform,
		private poiService: PoiService,
		private utilityService: UtilityService,
		private globalService: GlobalService,
		private geolocation: Geolocation
	) {
		this.checkPlatform();
		this.setUtility();
		this.getCategoriePoi();
		this.platform.pause.subscribe(() => {
			this.clearMapInterval();
		});

		this.platform.resume.subscribe(() => {
			this.clearMapInterval();
		});
	}

	ngOnInit() {}

	setUtility() {
		this.listaCategoriePoi = [];
		this.listaCategoriePoiSave = [];
		this.listaCategoriePoiFRR = [];
	}

	ionViewWillEnter() {
		this.utilityService.updateSlidePoi$.next(true);
		this.showMap = true;
		this.initializeMap();
	}

	checkPlatform() {
		this.ios = this.platform.is('ios');
		this.android = this.platform.is('android');
	}

	initializeMap() {
		setTimeout(() => {
			this.platform.ready().then(() => {
				initMap(this.geolocation, (msg) => {
					this.globalService.presentToast(msg, DURATIONTOAST.default);
				});
			});
		}, DELAYMAP.delayOne);
	}

	getCategoriePoi() {
		this.poiService.getLocalCategoriePoi().subscribe({
			next: (result) => {
				this.setCategoriesPoi(result);
				this.getRemoteCategoriePoi();
			},
			error: (error) => {
				this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
			},
		});
	}

	filterFromCat(categoria: CategoriaPoi) {
		this.categoriaSelected = categoria;
		if (categoria.b_selected) {
			clearLayers();
			geocodeRestSearch(null, categoria.nCatPoi);
		} else {
			clearLayers();
		}
	}

	searchCatPoiByTitle(titleCat: string) {
		this.listaCategoriePoi = this.listaCategoriePoiSave.filter((categoria) => categoria.tCatPoi.toLocaleUpperCase().includes(titleCat.toLocaleUpperCase()));
		this.listaCategoriePoiFRR = this.globalService.compositeForRowCatPoi(this.listaCategoriePoi, CATPOI.nForRow, CATPOI.maxRowFS);
	}

	ionViewWillLeave() {
		this.clearMapInterval();
		getWatchPositionRef()?.unsubscribe();
		this.showMap = false;
	}

	clearMapInterval() {
		const intervalRef = getLocationIntervalRef();
		if (intervalRef) {
			intervalRef.forEach((ir) => {
				clearInterval(ir);
			});
		}
	}

	setCategoriesPoi(categoriesPoi: CategoriaPoi[]) {
		this.listaCategoriePoi = categoriesPoi;
		this.listaCategoriePoiFRR = this.globalService.compositeForRowCatPoi(this.listaCategoriePoi, CATPOI.nForRow, CATPOI.maxRowFS);
		this.listaCategoriePoiSave = categoriesPoi;
	}

	getRemoteCategoriePoi() {
		this.poiService.getCategoriePoi().subscribe({
			next: (result) => {
				this.setCategoriesPoi(result.data);
			},
			error: (error) => {
				//this.globalService.presentToast(MESSAGES.serviceUnavailable, DURATIONTOAST.default);
			},
		});
	}
}
